@@ -2,34 +2,36 @@
 # this repository contains the full copyright notices and license terms.
 import csv
 import unicodedata
-from io import StringIO
-
 from collections import defaultdict
-from dateutil.relativedelta import relativedelta
 from decimal import Decimal
+from io import BytesIO, TextIOWrapper
 from operator import attrgetter
 
-from sql import Cast, Null, Literal
+import phonenumbers
+from dateutil.relativedelta import relativedelta
+# XXX fix: https://genshi.edgewall.org/ticket/582
+from genshi.template.astutil import ASTCodeGenerator, ASTTransformer
+from phonenumbers import NumberParseException
+from sql import Literal, Null
 from sql.aggregate import Count, Min, Sum
 from sql.conditionals import Case, Coalesce
-from sql.functions import Substring, Position, Extract, CurrentTimestamp
+from sql.functions import CurrentTimestamp, Extract
 from sql.operators import Exists
 
 from trytond.i18n import gettext
-from trytond.model import ModelSQL, ModelView, fields
-from trytond.model.modelsql import convert_from
+from trytond.model import ModelSQL, ModelView, convert_from, fields
+from trytond.modules.account.exceptions import FiscalYearNotFoundError
+from trytond.modules.account_eu.account import ECSalesList, ECSalesListContext
+from trytond.modules.currency.fields import Monetary
 from trytond.pool import Pool
 from trytond.pyson import Eval, If
 from trytond.report import Report
 from trytond.transaction import Transaction
-from trytond.wizard import Wizard, StateView, StateTransition, StateReport, \
-    Button
-from trytond.modules.account_eu.account import ECSalesList, ECSalesListContext
+from trytond.wizard import (
+    Button, StateReport, StateTransition, StateView, Wizard)
 
 from .exceptions import PrintError
 
-# XXX fix: https://genshi.edgewall.org/ticket/582
-from genshi.template.astutil import ASTCodeGenerator, ASTTransformer
 if not hasattr(ASTCodeGenerator, 'visit_NameConstant'):
     def visit_NameConstant(self, node):
         if node.value is None:
@@ -68,6 +70,17 @@ def format_percentage(n, size=5):
     return ('{0:.2f}'.format(n)).replace('.', '').rjust(size, '0')
 
 
+def format_phone(phone):
+    try:
+        phone = phonenumbers.format_number(
+            phonenumbers.parse(phone, 'ES'),
+            phonenumbers.PhoneNumberFormat.NATIONAL)
+    except NumberParseException:
+        phone = ''
+    phone = phone.replace(' ', '')
+    return phone[:9].rjust(9, '0')
+
+
 def identifier_code(identifier):
     if identifier:
         return identifier.es_code()
@@ -140,6 +153,7 @@ class AEATPartyReport(AEATReport):
         '''
         pool = Pool()
         Invoice = pool.get('account.invoice')
+        Move = pool.get('account.move')
 
         table, _ = tables[None]
         is_invoice = table.origin.like(Invoice.__name__ + ',%')
@@ -150,10 +164,8 @@ class AEATPartyReport(AEATReport):
             invoice = Invoice.__table__()
             tables['invoice'] = {
                 None: (invoice, (is_invoice
-                        & (invoice.id == Cast(
-                                Substring(table.origin,
-                                    Position(',', table.origin) + Literal(1)),
-                                Invoice.id.sql_type().base)))),
+                        & (invoice.id == Move.origin.sql_id(
+                                table.origin, Invoice)))),
                 }
 
         return Case((is_invoice, invoice.party), else_=Null), tables
@@ -380,27 +392,16 @@ class ESVATList(ModelSQL, ModelView):
     province_code = fields.Function(fields.Char("Province Code"),
         'get_province_code', searcher='search_province_code')
     code = fields.Char("Code")
-    amount = fields.Numeric(
-        "Amount", digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    first_period_amount = fields.Numeric(
-        "First Period Amount", digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    second_period_amount = fields.Numeric(
-        "Second Period Amount", digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    third_period_amount = fields.Numeric(
-        "Third Period Amount", digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    fourth_period_amount = fields.Numeric(
-        "Fourth Period Amount", digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
+    amount = Monetary("Amount", currency='currency', digits='currency')
+    first_period_amount = Monetary(
+        "First Period Amount", currency='currency', digits='currency')
+    second_period_amount = Monetary(
+        "Second Period Amount", currency='currency', digits='currency')
+    third_period_amount = Monetary(
+        "Third Period Amount", currency='currency', digits='currency')
+    fourth_period_amount = Monetary(
+        "Fourth Period Amount", currency='currency', digits='currency')
     currency = fields.Many2One('currency.currency', "Currency")
-    currency_digits = fields.Function(
-        fields.Integer("Currency Digits"), 'get_currency_digits')
-
-    def get_currency_digits(self, name):
-        return self.currency.digits
 
     @classmethod
     def get_province_code(cls, records, name):
@@ -554,6 +555,7 @@ class AEAT347(Report):
                 15, '0')
         context['format_decimal'] = format_decimal
         context['format_integer'] = format_integer
+        context['format_phone'] = format_phone
         context['identifier_code'] = identifier_code
         context['country_code'] = country_code
         context['strip_accents'] = strip_accents
@@ -643,16 +645,14 @@ class ECOperationListContext(ECSalesListContext):
                 ('start_date', '<=', Eval('end_date')),
                 (),
                 ),
-            ],
-        depends=['end_date'])
+            ])
     end_date = fields.Date("End Date",
         domain=[
             If(Eval('start_date'),
                 ('end_date', '>=', Eval('start_date')),
                 (),
                 ),
-            ],
-        depends=['start_date'])
+            ])
 
     @classmethod
     def default_start_date(cls):
@@ -712,6 +712,7 @@ class AEAT349(Report):
             return ('{0:.2f}'.format(abs(n))).replace('.', '').rjust(
                 digits, '0')
         context['format_decimal'] = format_decimal
+        context['format_phone'] = format_phone
 
         return context
 
@@ -725,19 +726,17 @@ class ESVATBookContext(ModelView):
         required=True,
         domain=[
             ('company', '=', Eval('company')),
-            ],
-        depends=['company'])
+            ])
     start_period = fields.Many2One('account.period', "Start Period",
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('start_date', '<=', (Eval('end_period'), 'start_date')),
-            ], depends=['fiscalyear', 'end_period'])
+            ])
     end_period = fields.Many2One('account.period', "End Period",
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('start_date', '>=', (Eval('start_period'), 'start_date'))
-            ],
-        depends=['fiscalyear', 'start_period'])
+            ])
     es_vat_book_type = fields.Selection([
             # Use same key as tax authority
             ('E', "Issued"),
@@ -757,8 +756,13 @@ class ESVATBookContext(ModelView):
     @classmethod
     def default_fiscalyear(cls):
         pool = Pool()
-        Fiscalyear = pool.get('account.fiscalyear')
-        return Fiscalyear.find(cls.default_company(), exception=False)
+        FiscalYear = pool.get('account.fiscalyear')
+        try:
+            fiscalyear = FiscalYear.find(
+                cls.default_company(), test_state=False)
+        except FiscalYearNotFoundError:
+            return None
+        return fiscalyear.id
 
 
 class ESVATBook(ModelSQL, ModelView):
@@ -771,21 +775,20 @@ class ESVATBook(ModelSQL, ModelView):
     party_tax_identifier = fields.Many2One(
         'party.identifier', "Party Tax Identifier")
     tax = fields.Many2One('account.tax', "Tax")
-    base_amount = fields.Numeric("Base Amount",
-        digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    tax_amount = fields.Numeric("Tax Amount",
-        digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
+    base_amount = Monetary(
+        "Base Amount", currency='currency', digits='currency')
+    tax_amount = Monetary(
+        "Tax Amount", currency='currency', digits='currency')
     surcharge_tax = fields.Many2One('account.tax', "Surcharge Tax")
-    surcharge_tax_amount = fields.Numeric("Surcharge Tax Amount",
-        digits=(16, Eval('currency_digits', 2)),
+    surcharge_tax_amount = Monetary(
+        "Surcharge Tax Amount", currency='currency', digits='currency',
         states={
             'invisible': ~(Eval('surcharge_tax', None)),
-            },
-        depends=['currency_digits', 'surcharge_tax'])
-    currency_digits = fields.Function(fields.Integer("Currency Digits"),
-        'get_currency_digits')
+            })
+
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"),
+        'get_currency')
 
     @classmethod
     def included_tax_groups(cls):
@@ -891,8 +894,8 @@ class ESVATBook(ModelSQL, ModelView):
                     ]))
         return query
 
-    def get_currency_digits(self, name):
-        return self.invoice.company.currency.digits
+    def get_currency(self, name):
+        return self.invoice.company.currency.id
 
 
 class VATBookReport(Report):
@@ -939,16 +942,14 @@ class VATBookReport(Report):
 
     @classmethod
     def render_csv(cls, report, report_context):
-        vat_book = StringIO()
+        vat_book = BytesIO()
         writer = csv.writer(
-            vat_book, delimiter=';', doublequote=False, escapechar='\\',
+            TextIOWrapper(vat_book, encoding='utf-8', write_through=True),
+            delimiter=';', doublequote=False, escapechar='\\',
             quoting=csv.QUOTE_NONE)
         for record in report_context['records']:
             writer.writerow(cls.get_row(record, report_context))
-        value = vat_book.getvalue()
-        if not isinstance(value, bytes):
-            value = value.encode('utf-8')
-        return value
+        return vat_book.getvalue()
 
     @classmethod
     def get_row(cls, record, report_context):
